import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  max
} from 'd3';
const svg = select('svg'); 
const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => { //To render a React element into a root DOM node
  const title = 'Population of India (1955-2020) - Line Chart'; 
  
  const xValue = d => +d.Year; 
  const xAxisLabel = 'Year'; 

  const yValue = d => d.Population; 
  const yAxisLabel = 'Population'; 
  
  const margin = { top: 60, right: 40, bottom: 88, left: 150 };// providing the margin for line chart using css properties
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  //container
  
  const xScale = scaleTime()
  //complete set of values
    .domain(extent(data, xValue)) //returns the minimum and maximum value in an array from the given array using natural order.
    .range([0,innerWidth]) //output bounds
    .nice(); // it will round the values out
  
  const yScale = scaleLinear() // 
    .domain([0, max(data, yValue )])
    .range([innerHeight,0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);//center g in svg
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight) // to get axis position at bottom
    .tickPadding(20);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth) // to get axis position at left side
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis); //It appends a 'g' element to the SVG. g element is used to group SVG shapes together,
  	yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text') // for labeling the text
      .attr('class', 'axis-label')
      .attr('y', -90)
      .attr('x', -innerHeight / 3)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    	.attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text') // for x axis labeling
      .attr('class', 'axis-label')
      .attr('y', 70)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  const lineGenerator = line() // it generrates horizantal and vertical lines for x and y axis
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d))); //coordinates in pixels
  
  g.append('path') // provides path to the line 
  	.attr('class', 'line_path')
  	.attr('d', lineGenerator(data))
    console.log(lineGenerator(data))
  
  g.append('text') // for labeling the title
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
};
// getting the csv data uisng github url link for my data 
csv('https://gist.githubusercontent.com/manasanimmagadda13/5de9a1f7a2b82c0d9d0a242f8f087fea/raw/a342c786f840d8ae23141ec6f7e320f8d53433c4/indiaPopulation1955-2020.csv')
  .then(data => {
  	//console.log(data)
    data.forEach(d => {
    	d.Population = +d.Population;
      d.Year = new Date(d.Year)
    });
  console.log(data)
    render(data);
  });